// ConeWS.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"
#include <array>
#include <iostream>
#include <cmath>
#include <cfloat>
#include <chrono>
//#include <windows.h>

#define PI 3.14159265
using namespace std;
using namespace std::chrono;


typedef tr1::array<double, 3>  coord;
bool epsCompar(double a, double b)
{
	if (abs(a - b) < 0.000001)
	{
		//absolutni chyba pro hodnoty blizke nule
		return true;
	}
	if (abs((a - b) / max(abs(a), abs(b))) < DBL_EPSILON)
	{
		return true;
	}
	return false;
}
coord crossProduct(coord X, coord Y, coord Z)
{
	coord v1 = { { X[0] - Y[0], X[1] - Y[1], X[2] - Y[2] } };
	coord v2 = { { Z[0] - Y[0], Z[1] - Y[1], Z[2] - Y[2] } };
	coord crossProduct;

	crossProduct[0] = v1[1] * v2[2] - v1[2] * v2[1];
	crossProduct[1] = v1[2] * v2[0] - v1[0] * v2[2];
	crossProduct[2] = v1[0] * v2[1] - v1[1] * v2[0];

	return crossProduct;
}
coord crossProductVect(coord v1, coord v2)
{
	coord crossProduct;
	crossProduct[0] = v1[1] * v2[2] - v1[2] * v2[1];
	crossProduct[1] = v1[2] * v2[0] - v1[0] * v2[2];
	crossProduct[2] = v1[0] * v2[1] - v1[1] * v2[0];

	return crossProduct;
}
coord countNormal(coord X, coord Y, coord Z, bool& error)
{
	coord normal;
	double lenght;
	normal = crossProduct(X, Y, Z);
	lenght = sqrt(normal[0] * normal[0] + normal[1] * normal[1] + normal[2] * normal[2]);

	if (epsCompar(lenght, 0.0))
	{
		error = true;
	}
	else
	{
		normal[0] = normal[0] / lenght;
		normal[1] = normal[1] / lenght;
		normal[2] = normal[2] / lenght;

		error = false;
		return normal;
	}

	coord zeros = { { 0.0,0.0,0.0 } };
	return zeros;
}
tr1::array<double, 4> planeEqv(coord A, coord B, coord C, bool& error)
{
	coord normal;
	tr1::array<double, 4> planeEquation;
	normal = countNormal(A, B, C, error);
	planeEquation[0] = normal[0];
	planeEquation[1] = normal[1];
	planeEquation[2] = normal[2];
	planeEquation[3] = -(normal[0] * A[0] + normal[1] * A[1] + normal[2] * A[2]);
	return planeEquation;
}
bool getRotMatrix(tr1::array<double, 3> z0, tr1::array<double, 3> z1, tr1::array<tr1::array<double, 3>, 3>& RM)
{
	//uhel mezi vektory
	double alfa = acos(z0[0]*z1[0]+ z0[1] * z1[1]+ z0[2] * z1[2]);

	if (epsCompar(alfa, 0.0))
	{
		return false;
	}
	else
	{
		coord rot = crossProductVect(z0, z1);
		double lenght = sqrt(rot[0] * rot[0] + rot[1] * rot[1] + rot[2] * rot[2]);
		coord rotAxis = { { rot[0] / lenght, rot[1] / lenght, rot[2] / lenght } };

		tr1::array<double, 4>quater = { { cos(alfa / 2), sin(alfa / 2)*rotAxis[0], sin(alfa / 2)*rotAxis[1], sin(alfa / 2)*rotAxis[2] } };

		RM[0][0] = 1 - 2 * quater[2] * quater[2] - 2 * quater[3] * quater[3];
		RM[1][0] = 2 * quater[1] * quater[2] + 2 * quater[3] * quater[0];
		RM[2][0] = 2 * quater[1] * quater[3] - 2 * quater[2] * quater[0];

		RM[0][1] = 2 * quater[1] * quater[2] - 2 * quater[3] * quater[0];
		RM[1][1] = 1 - 2 * quater[1] * quater[1] - 2 * quater[3] * quater[3];
		RM[2][1] = 2 * quater[2] * quater[3] + 2 * quater[1] * quater[0];

		RM[0][2] = 2 * quater[1] * quater[3] + 2 * quater[2] * quater[0];
		RM[1][2] = 2 * quater[2] * quater[3] - 2 * quater[1] * quater[0];
		RM[2][2] = 1 - 2 * quater[1] * quater[1] - 2 * quater[2] * quater[2];

		return true;
	}

}
coord rotation(coord P, tr1::array<tr1::array<double, 3>, 3>& RM, bool rotDir)
{
	coord R;
	if (!rotDir)
	{
		R[0] = P[0] * RM[0][0] + P[1] * RM[1][0] + P[2] * RM[2][0];
		R[1] = P[0] * RM[0][1] + P[1] * RM[1][1] + P[2] * RM[2][1];
		R[2] = P[0] * RM[0][2] + P[1] * RM[1][2] + P[2] * RM[2][2];
	}
	else
	{
		R[0] = P[0] * RM[0][0] + P[1] * RM[0][1] + P[2] * RM[0][2];
		R[1] = P[0] * RM[1][0] + P[1] * RM[1][1] + P[2] * RM[1][2];
		R[2] = P[0] * RM[2][0] + P[1] * RM[2][1] + P[2] * RM[2][2];
	}
	return R;
}
void intersection(coord V, coord R, tr1::array<double, 4>planeEquation, int& position, coord& interPoint)
{
	//rovnobezna, stejna
	double t;
	coord u = { { V[0] - R[0],V[1] - R[1],V[2] - R[2] } };
	double par = { u[0] * planeEquation[0] + u[1] * planeEquation[1] + u[2] * planeEquation[2] };

	//primka je rovnobezna s rovinou, nebo v ni lezi?
	if (epsCompar(par, 0.0))
	{
		//lezi v rovine
		if (epsCompar(V[0] * planeEquation[0] + V[1] * planeEquation[1] + V[2] * planeEquation[2] + planeEquation[3], 0.0))
		{
			position = 0;
		}
		//je rovnobezna
		else
		{
			position = -1;
		}
	}
	//ruznobezna->hledame prusecik
	else
	{
		t = -((planeEquation[0] * R[0] + planeEquation[1] * R[1] + planeEquation[2] * R[2] + planeEquation[3]) / (planeEquation[0] * u[0] + planeEquation[1] * u[1] + planeEquation[2] * u[2]));
		interPoint[0] = R[0] + u[0] * t;
		interPoint[1] = R[1] + u[1] * t;
		interPoint[2] = R[2] + u[2] * t;
		position = 1;
	}
}
void normalize(coord& u)
{
	double lenght = sqrt(u[0] * u[0] + u[1] * u[1] + u[2] * u[2]);
	u[0] = u[0] / lenght;
	u[1] = u[1] / lenght;
	u[2] = u[2] / lenght;
}
double distancePP(coord A, coord B, coord& u)
{
	u[0] = A[0] - B[0];
	u[1] = A[1] - B[1];
	u[2] = A[2] - B[2];
	double lenght = sqrt(u[0] * u[0] + u[1] * u[1] + u[2] * u[2]);
	normalize(u);
	return lenght;
}
double distanceLP(coord L1, coord L2, coord S)
{
	coord u = { { L1[0] - L2[0],L1[1] - L2[1],L1[2] - L2[2] } };
	double t = (u[0] * (S[0] - L1[0]) + u[1] * (S[1] - L1[1]) + u[2] * (S[2] - L1[2])) / (u[0] * u[0] + u[1] * u[1] + u[2] * u[2]); //=0
	coord X = { { L1[0] + t*u[0],L1[1] + t*u[1] ,L1[2] + t*u[2] } };
	return distancePP(X, S, u);
}
void interLR(coord& interPoint1, coord& interPoint2, coord S, double r)
{
	coord u;
	if (epsCompar(interPoint1[0], interPoint2[0])&& epsCompar(interPoint1[1], interPoint2[1])&& epsCompar(interPoint1[2], interPoint2[2]))
	{
		//body se rovnaji, posuneme, aby vznikl vektor
		u = { { 0.1,0.1,0.1 } };

	}
	//prusecik primky a kruznice
	else
	{
		u = { { interPoint1[0] - interPoint2[0],interPoint1[1] - interPoint2[1],interPoint1[2] - interPoint2[2] } };
	}
	

	coord coef = { { u[0] * u[0] + u[1] * u[1],(interPoint1[0] - S[0])* u[0] * 2 + (interPoint1[1] - S[1])* u[1] * 2,(interPoint1[0] - S[0])*(interPoint1[0] - S[0]) + (interPoint1[1] - S[1])*(interPoint1[1] - S[1]) - r*r } };
	double dis = coef[1] * coef[1] - 4 * coef[0] * coef[2];
	double t1 = (-coef[1] - sqrt(dis)) / (2 * coef[0]);
	double t2 = (-coef[1] + sqrt(dis)) / (2 * coef[0]);


	interPoint2[0] = interPoint1[0] + t2*u[0];
	interPoint2[1] = interPoint1[1] + t2*u[1];
	interPoint2[2] = interPoint1[2];
	interPoint1[0] = interPoint1[0] + t1*u[0];
	interPoint1[1] = interPoint1[1] + t1*u[1];
	interPoint1[2] = interPoint1[2];
}
bool comparison(coord interPoint1, coord interPoint2, coord A, coord B)
{
	//lezi zadane dva body A,B mezi pruseciky
	for (int i = 0; i < 3; i++)
	{
		if ((interPoint1[i] < interPoint2[i]))
		{
			if (((A[i] > interPoint1[i]) || epsCompar(A[i], interPoint1[i])) && ((A[i]< interPoint2[i]) || epsCompar(A[i], interPoint2[i])) && ((B[i]>interPoint1[i]) || epsCompar(B[i], interPoint1[i])) && (B[i] < interPoint2[i]) || epsCompar(B[i], interPoint2[i]))
			{
			}
			else
			{
				return false;
			}
		}
		else if (interPoint1[i] > interPoint2[i])
		{
			if (((A[i] < interPoint1[i]) || epsCompar(A[i], interPoint1[i])) && ((A[i]> interPoint2[i]) || epsCompar(A[i], interPoint2[i])) && ((B[i]<interPoint1[i]) || epsCompar(B[i], interPoint1[i])) && (B[i] > interPoint2[i]) || epsCompar(B[i], interPoint2[i]))
			{
			}
			else
			{
				return false;
			}
		}
	}
	return true;
}
coord interLL(coord A, coord u, coord B, coord v, bool& e)
{
	e = false;
	double d = (v[1] - (v[0] * u[1] / u[0]));
	double s;
	coord interPoint;
	if (epsCompar(d, 0.0))
	{
		d = (v[2] - (v[0] * u[2] / u[0]));
		if (epsCompar(d, 0.0))
		{
			s = (A[2] - B[2] + ((u[2] / u[1])*(B[1] - A[1]))) / (v[2] - (v[1] * u[2] / u[1]));
			interPoint = { B[0] + s*v[0],B[1] + s*v[1] ,B[2] + s*v[2] };
		}
		else
		{
			s = (A[2] - B[2] + ((u[2] / u[0])*(B[0] - A[0]))) / (v[2] - (v[0] * u[2] / u[0]));
			interPoint = { B[0] + s*v[0],B[1] + s*v[1] ,B[2] + s*v[2] };
		}
	}
	else
	{
		s = (A[1] - B[1] + ((u[1] / u[0])*(B[0] - A[0]))) / (v[1] - (v[0] * u[1] / u[0]));
		interPoint = { B[0] + s*v[0],B[1] + s*v[1] ,B[2] + s*v[2] };
	}
	if (isnan(s) || isinf(s))
	{
		e = true;
	}
	return interPoint;
}
bool inputCheck(coord V, coord S, coord X, coord Y, tr1::array<double, 4>& SXY)
{
	bool err = false;
	coord u;

	// a) SXY tvori rovinu (jsou ruzne)
	SXY = planeEqv(S, X, Y, err);
	if (err)
	{
		//cout << "CHYBA: Body X, Y a S nelezi v rovine" << endl;
	}

	// b) X, Y lezi na kruznici:
	/*if (!epsCompar(distancePP(X, S, u), distancePP(Y, S, u)))
	{
		//cout << "CHYBA: Body X a Y nemaji stejnou vzdalenost od S, tedy netvori kruhovou podstavu" << endl;
		err = true;
	}*/

	// c) vrchol V lezi v rovine podstavy
	double konst = V[0] * SXY[0] + V[1] * SXY[1] + V[2] * SXY[2] + SXY[3];

	if (epsCompar(konst, 0.0))
	{
		//cout << "CHYBA: vrchol V lezi v rovine podstavy" << endl;
		err = true;
	}
	return err;
}
void rescale(coord V, coord S, coord& Sn, coord& Vn, double& r, double ra, coord& Xn, coord& Yn)
{
	coord u;
	coord v;
	coord w;


	coord X = { { r,0.0,0.0 } };
	coord Y = { { -r,0.0,0.0 } };
	distancePP(X, Y, u);
	distancePP(V, Y, v);
	distancePP(X, V, w);

	// n -kolmice na strany trojuhelniku -PPR -> smer dovnitr
	coord n1 = crossProductVect(u, v);
	n1 = crossProductVect(v, n1);

	v[0] = -v[0];
	v[1] = -v[1];
	v[2] = -v[2];
	coord n2 = crossProductVect(v, w);
	n2 = crossProductVect(w, n2);

	u[0] = -u[0];
	u[1] = -u[1];
	u[2] = -u[2];
	w[0] = -w[0];
	w[1] = -w[1];
	w[2] = -w[2];

	coord n3 = crossProductVect(w, u);
	n3 = crossProductVect(u, n3);

	normalize(n1);
	normalize(n2);
	normalize(n3);

	//ve smeru kolmic posouvame bod na primce o vydalenost r
	X[0] = X[0] + ra*n2[0];
	X[1] = X[1] + ra*n2[1];
	X[2] = X[2] + ra*n2[2];
	Y[0] = Y[0] + ra*n3[0];
	Y[1] = Y[1] + ra*n3[1];
	Y[2] = Y[2] + ra*n3[2];
	V[0] = V[0] + ra*n1[0];
	V[1] = V[1] + ra*n1[1];
	V[2] = V[2] + ra*n1[2];

	//pruseciky novych primek = nove body zmenseneho kuzelu
	bool e;
	Xn = interLL(X, w, Y, u,e);
	Yn = interLL(V, v, Y, u,e);
	Vn = interLL(V, v, X, w,e);
	Sn[0] = Yn[0] + ((Xn[0] - Yn[0]) / 2);
	Sn[1] = Yn[1] + ((Xn[1] - Yn[1]) / 2);
	Sn[2] = Xn[2];
	r = distancePP(Sn, Xn, u);
}
void EuAngl(coord A, coord B, double& rotX, double& rotY, double& rotZ)
{
	coord p = { { B[0] - A[0],B[1] - A[1],B[2] - A[2] } };
	double l = sqrt(p[0] * p[0] + p[1] * p[1] + p[2] * p[2]);
	p[0] = p[0] / l; p[1] = p[1] / l; p[2] = p[2] / l;

	tr1::array<double, 3> rotAxis;
	tr1::array<tr1::array<double, 3>, 3> RM2;
	tr1::array<double, 4> vec = { { rotAxis[0],rotAxis[1],rotAxis[2],0 } };
	coord z0 = { { 0.0,0.0,1.0 } };
	getRotMatrix(p,z0, RM2);

	if (!epsCompar(RM2[2][0], 1.0) && !epsCompar(RM2[2][0], -1.0))
	{
		rotY = -asin(RM2[2][0]);
		rotX = atan2(RM2[2][1] / cos(rotY), RM2[2][2] / cos(rotY));
		rotZ = atan2(RM2[1][0] / cos(rotY), RM2[0][0] / cos(rotY));
	}
	else
	{
		rotZ = 0.0;
		if (epsCompar(RM2[2][0], -1.0))
		{
			rotY = PI / 2;
			rotX = rotZ + atan2(RM2[0][1], RM2[0][2]);
		}
		else
		{
			rotY = -PI / 2;
			rotX = -rotZ + atan2(-RM2[0][1], -RM2[0][2]);
		}
	}
	rotX = rotX*180.0 / PI;
	rotY = rotY*180.0 / PI;
	rotZ = rotZ*180.0 / PI;
}
int main()
{
	// 1) ZADANI VSTUPNICH DAT:  vrchol+ stred podstavy + 2 ruzne body na kruznici
	time_point<steady_clock> start, finish;
	
	bool result = true;
	//vrchol
	coord V = { { 1.0,1.0,4.0 } };

	//stred kruznice
	coord So = { { 0.0,0.0,0.0 } };

	//body na kruznici
	//zadvat s vysokou pesnost!!!!
	//coord X = { { sqrt(1600 - (23.88 +3.0)*(23.88 +3.0) - (31.57 - 2.0)*(31.57 - 2.0)) + 2.0,23.88,31.57 } };
	//coord Y = { { sqrt(1600 - (16.3 + 3.0)*(16.3 +3.0) - (33.95 - 2.0)*(33.95 - 2.0)) + 2.0,16.3,33.95 } };
	coord X = { { -0.66,-2.8,0.86 } };
	coord Y = { {-2.0,-2.0,1.0} };

	//cout << "puvodni souradnice kuzelu:" << endl;
	//cout << "X: " << X[0] << " " << X[1] << " " << X[2] << endl;
	//cout << "Y: " << Y[0] << " " << Y[1] << " " << Y[2] << endl;
	//cout << "V: " << V[0] << " " << V[1] << " " << V[2] << endl;
	//cout <<  endl;

	// 2) KONTROLA VSTUPNICH DAT

	tr1::array<double, 4> SXY;

	if (inputCheck(V, So, X, Y, SXY))
	{
		//cout << "Chyba ve vstupnich datech." << endl;
		result = false;
	}
	else
	{
		// 3) TRANSORMACE => S=[0,0,0], kruznice v rovine XY
		// a) posunuti => vsechny body -S
		for (int i = 0; i < 3; i++)
		{
			V[i] = V[i] - So[i];
			X[i] = X[i] - So[i];
			Y[i] = Y[i] - So[i];
		}
		coord S = { { 0.0,0.0,0.0 } };

		// b) rotace 


		bool err;
		SXY = planeEqv(S, X, Y, err);
		tr1::array<tr1::array<double, 3>, 3> RM;
		coord z0 = { { 0.0,0.0,1.0 } };

		bool rotate = getRotMatrix(z0, {SXY[0], SXY[1], SXY[2]}, RM);
		bool rotDir = signbit(SXY[2]);
		if (rotate)
		{
			V = rotation(V, RM, rotDir);
			X = rotation(X, RM, rotDir);
			Y = rotation(Y, RM, rotDir);
		}
		
		start = high_resolution_clock::now();
		// 4) ZADAVANI USECEK
		coord A = { {0.19,0.19,0.76} };
		coord B = { {0.38, 0.38, 1.51 } };
		double ra = 0.2;
		double VisX = A[0];
		double VisY = A[1];
		double VisZ = A[2];
		double VisRollX, VisRollY, VisRollZ;
		EuAngl(A, B, VisRollX, VisRollY, VisRollZ);
		coord vec;
		double l = distancePP(A, B, vec);
		//cout << "puvodni usecka:"<<endl;
		//cout << "A: " << A[0] << " " << A[1] << " " << A[2] << endl;
		//cout << "B: " << B[0] << " " << B[1] << " " << B[2] << endl;
		//cout << "delka: " << l << endl;
		//cout << endl;

		//4.1) kontrola
		if (epsCompar(A[0], B[0]) && epsCompar(A[1], B[1]) && epsCompar(A[2], B[2]))
		{
			//cout << "CHYBA: zadane body netvori usecku" << endl;
		}
		else
		{
			// 4.2) transformace usecky
			for (int i = 0; i < 3; i++)
			{
				A[i] = A[i] - So[i];
				B[i] = B[i] - So[i];
			}
			if (rotate)
			{
				A = rotation(A, RM, rotDir);
				B = rotation(B, RM, rotDir);
			}
			//cout << "rotovane  souradnice:" << endl;
			//cout << "X: " << X[0] << " " << X[1] << " " << X[2] << endl;
			//cout << "Y: " << Y[0] << " " << Y[1] << " " << Y[2] << endl;
			//cout << "V: " << V[0] << " " << V[1] << " " << V[2] << endl;
			//cout << "S: " << S[0] << " " << S[1] << " " << S[2] << endl;
			//cout << "A: " << A[0] << " " << A[1] << " " << A[2] << endl;
			//cout << "B: " << B[0] << " " << B[1] << " " << B[2] << endl;
			//cout << endl;

			// 5) ZMENSENI KUZELU O POLOMER
			coord u, Sn, Vn, Xn, Yn;
			double r = distancePP(S, X, u);
			coord coord_r = { { r,0.0,0.0 } };
			coord coord_nr = { { -r,0.0,0.0 } };
			if (ra < min(min(distancePP(V, coord_r, u), distancePP(V, coord_nr, u)), 2 * r))
			{

				rescale(V, S, Sn, Vn, r, ra, Xn, Yn);
				//cout << "zmenseny kuzel:" << endl;
				//cout << "X: " << Xn[0] << " " << Xn[1] << " " << Xn[2] << endl;
				//cout << "Y: " << Yn[0] << " " << Yn[1] << " " << Yn[2] << endl;
				//cout << "V: " << Vn[0] << " " << Vn[1] << " " << Vn[2] << endl;
				//cout << "S: " << Sn[0] << " " << Sn[1] << " " << Sn[2] << endl;

				if (signbit(V[2]))
				{
					if (!(Vn[2] < Sn[2]))
					{
						result = false;
					}
					else if (Xn[0]< Yn[0])
					{
						result = false;
					}
				}
				else
				{
					if (Vn[2] < Sn[2])
					{
						result = false;
					}
					else if (Xn[0]< Yn[0])
					{
						result = false;
					}
				}

			}
			else
			{
				result = false;
			}
			// posunuti zpet do poc. souradnic
			V[0] = Vn[0] - Sn[0]; V[1] = Vn[1] - Sn[1]; V[2] = Vn[2] - Sn[2];
			A[0] = A[0] - Sn[0]; A[1] = A[1] - Sn[1]; A[2] = A[2] - Sn[2];
			B[0] = B[0] - Sn[0]; B[1] = B[1] - Sn[1]; B[2] = B[2] - Sn[2];
			X[0] = Xn[0] - Sn[0];
			X[1] = Xn[1] - Sn[1];
			X[2] = Xn[2] - Sn[2];
			Y[0] = Yn[0] - Sn[0];
			Y[1] = Yn[1] - Sn[1];
			Y[2] = Yn[2] - Sn[2];

			//cout << endl;
			//cout << "vysledne souradnice:" << endl;
			//cout << "X: " << X[0] << " " << X[1] << " " << X[2] << endl;
			//cout << "Y: " << Y[0] << " " << Y[1] << " " << Y[2] << endl;
			//cout << "V: " << V[0] << " " << V[1] << " " << V[2] << endl;
			//cout << "A: " << A[0] << " " << A[1] << " " << A[2] << endl;
			//cout << "B: " << B[0] << " " << B[1] << " " << B[2] << endl;
			//cout << "S: " << S[0] << " " << S[1] << " " << S[2] << endl;

			// 4) VYPOCET
			tr1::array<double, 4> xyPlane = { { 0.0,0.0,1.0,0.0 } };

			if (signbit(A[2]) != signbit(V[2]))
			{
				result = false;
			}
			if (signbit(B[2]) != signbit(V[2]))
			{
				result = false;
			}
			//parametry pruseciku
			int pos; coord interPoint1, interPoint2;

			//vrcholova rovina - pokud prochazi vrcholem, tak prunik primky s podstavou !!!!!
			bool error = false;
			tr1::array<double, 4> vertexPlane = planeEqv(A, B, V, error);


			if (error == true && result)
			{
				intersection(A, B, xyPlane, pos, interPoint1);

				if (pos == -1)
				{
					//rovnobezna s podstavou+ prochazi vrcholem -> nema prusecik
					result = false;
				}
				else if (pos == 1)
				{
					//1 prusecik je vrchol, 2. je prusecik interPoint => vypocitavame, jestli lezi uvnitr kruznice (vzdalenost 2 bodu)
					coord v;
					if ((distancePP(interPoint1, S, v) < r) || epsCompar(distancePP(interPoint1, S, v), r))
					{
						//OK
						//porovnani, jestli lezi uvnitr pruseciku
						result = comparison(interPoint1, V, A, B);
					}
					else
					{
						result = false;
					}
				}
			}

			//pudorysny stopnik
			else if (result)
			{
				distancePP(A, B, u);
				coord R = { { A[0] + u[0],A[1] + u[1],A[2] + u[2] } };
				if (epsCompar(R[0], B[0]) && epsCompar(R[1], B[1]) && epsCompar(R[2], B[2]))
				{
					R[0] = R[0] + u[0];
					R[1] = R[1] + u[1];
					R[2] = R[2] + u[2];
				}
				intersection(B, A, xyPlane, pos, interPoint1);

				if (pos == 0)
				{
					if (distanceLP(A, B, S) < r || epsCompar(distanceLP(A, B, S), r))
					{
						interPoint1 = A;
						interPoint2 = B;
						interLR(interPoint1, interPoint2, S, r);
						//POROVNN
						result = comparison(interPoint1, interPoint2, A, B);
					}
					else
					{
						result = false;
					}

				}
				else if (pos == -1)
				{
					//rovnobezna s podstavou !!!! 
					//prepocitame polomer a stred noveho (posunuteho kruhu)
					double dist = distancePP(S, V, u);
					double k = A[2] / u[2];
					coord Sn = { { S[0] + u[0] * k, S[1] + u[1] * k, S[2] + u[2] * k } };
					double rn = (r*distancePP(Sn, V, u)) / dist;
					// pruseciky nove kruznice s primkou
					if (distanceLP(A, B, Sn) < rn || epsCompar(distanceLP(A, B, Sn), rn))
					{
						interLR(interPoint1, interPoint2, Sn, rn);
						//POROVNANI
						result = comparison(interPoint1, interPoint2, A, B);
					}
					else
					{
						result = false;
					}

				}
				else if (pos == 1)
				{
					//pudorysny stopnik AB - interPoint -V ==vrcholova rovina
					intersection(V, R, xyPlane, pos, interPoint2);
					//podle vydalenosti primky od bodu 
					double dist = distanceLP(interPoint1, interPoint2, S);
					if (dist < r)
					{
						//prusecik kruznice a primky
						interLR(interPoint1, interPoint2, S, r);
						//prusecik 2 primek
						coord v;
						distancePP(interPoint1, V, v);
						interPoint1 = interLL(V, v, A, u, err);
						if (err)
						{
							intersection(V, A, xyPlane, pos, interPoint1);
							if (!distancePP(interPoint1, S, v) < r)
							{
								result = false;
							}
						}

						distancePP(interPoint2, V, v);
						interPoint2 = interLL(V, v, A, u, err);
						if (err)
						{
							intersection(V, A, xyPlane, pos, interPoint2);
							if (!distancePP(interPoint2, S, v) < r)
							{
								result = false;
							}
						}

						//POROVNANI
						if (signbit(V[2]) && ((interPoint1[2] < V[2]) || (interPoint2[2] < V[2]) || (interPoint2[2] > S[2]) || (interPoint1[2] > S[2])))
						{
							if ((interPoint1[2] < V[2]) || (interPoint1[2] > S[2]))
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint1);
							}
							else
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint2);
							}
						}
						else if ((!signbit(V[2])) && ((interPoint1[2] > V[2]) || (interPoint2[2] > V[2]) || (interPoint2[2] < S[2]) || (interPoint1[2] < S[2])))
						{
							if ((interPoint1[2] > V[2]) || (interPoint1[2] < S[2]))
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint1);
							}
							else
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint2);
							}
						}
						else
						{
							result = comparison(interPoint1, interPoint2, A, B);
						}
					}
					else
					{
						result = false;
					}
				}
			}
		}
		//cout << "X: " << VisX << endl;
		//cout << "Y: " << VisY  << endl;
		//cout << "Z: " << VisZ << endl;
		//cout << "rotX: " << VisRollX << endl;
		//cout << "rotY: " << VisRollY << endl;
		//cout << "rotZ: " << VisRollZ << endl;
	}
	if (result)
	{
		cout << "usecka lezi uvnitr zadaneho kuzelu" << endl;
	}
	else
	{
		cout << "CHYBA: cast, nebo cela zadana usecka je mimo kuzel" << endl;
	}
	finish = high_resolution_clock::now();

	auto sec = duration_cast<microseconds>(finish - start);
	cout << "time: " << sec.count() << " us" << endl;
	cin.get();

	return 0;
}

