// ConeWorkSpace.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <array>
#include <iostream>
#include <cmath>
#include <cfloat>
#include <chrono>
//#include <windows.h>

using namespace std::chrono;
using namespace std;



typedef array<double, 3>  coord;
bool epsCompar(double a, double b)
{
	if (abs(a - b) < 0.000001)
	{
		//absolutni chyba pro hodnoty blizke nule
		return true;
	}
	if (abs((a - b) / max(abs(a), abs(b))) < DBL_EPSILON)
	{
		return true;
	}
	return false;
}
coord crossProduct(coord X, coord Y, coord Z)
{
	coord v1, v2, crossProduct;
	v1 = { X[0] - Y[0], X[1] - Y[1], X[2] - Y[2] };
	v2 = { Z[0] - Y[0], Z[1] - Y[1], Z[2] - Y[2] };

	crossProduct[0] = v1[1] * v2[2] - v1[2] * v2[1];
	crossProduct[1] = v1[2] * v2[0] - v1[0] * v2[2];
	crossProduct[2] = v1[0] * v2[1] - v1[1] * v2[0];

	return crossProduct;
}
coord crossProductVect(coord v1, coord v2)
{
	coord crossProduct;
	crossProduct[0] = v1[1] * v2[2] - v1[2] * v2[1];
	crossProduct[1] = v1[2] * v2[0] - v1[0] * v2[2];
	crossProduct[2] = v1[0] * v2[1] - v1[1] * v2[0];

	return crossProduct;
}
coord countNormal(coord X, coord Y, coord Z, bool& error)
{
	coord normal;
	double lenght;
	normal = crossProduct(X, Y, Z);
	lenght = sqrt(normal[0] * normal[0] + normal[1] * normal[1] + normal[2] * normal[2]);

	if (epsCompar(lenght, 0.0))
	{
		error = true;
	}
	else
	{
		normal[0] = normal[0] / lenght;
		normal[1] = normal[1] / lenght;
		normal[2] = normal[2] / lenght;

		error = false;
		return normal;
	}
	return{ 0.0,0.0,0.0 };
}
array<double, 4> planeEqv(coord A, coord B, coord C, bool& error)
{
	coord normal;
	array<double, 4> planeEquation;
	normal = countNormal(A, B, C, error);
	planeEquation[0] = normal[0];
	planeEquation[1] = normal[1];
	planeEquation[2] = normal[2];
	planeEquation[3] = -(normal[0] * A[0] + normal[1] * A[1] + normal[2] * A[2]);
	return planeEquation;
}
bool getRotMatrix(array<double, 3> z0, array<double, 3> z1, array<array<double, 3>, 3>& RM)
{
	//uhel mezi vektory
	double alfa = acos(z0[0]*z1[0]+ z0[1] * z1[1]+ z0[2] * z1[2]);

	if (epsCompar(alfa, 0.0))
	{
		return false;
	}
	else
	{
		coord rot = crossProductVect(z0, z1);
		double lenght = sqrt(rot[0] * rot[0] + rot[1] * rot[1] + rot[2] * rot[2]);
		coord rotAxis = { { rot[0] / lenght, rot[1] / lenght, rot[2] / lenght } };

		array<double, 4>quater = { cos(alfa / 2), sin(alfa / 2)*rotAxis[0], sin(alfa / 2)*rotAxis[1], sin(alfa / 2)*rotAxis[2] };

		RM[0][0] = 1 - 2 * quater[2] * quater[2] - 2 * quater[3] * quater[3];
		RM[1][0] = 2 * quater[1] * quater[2] + 2 * quater[3] * quater[0];
		RM[2][0] = 2 * quater[1] * quater[3] - 2 * quater[2] * quater[0];

		RM[0][1] = 2 * quater[1] * quater[2] - 2 * quater[3] * quater[0];
		RM[1][1] = 1 - 2 * quater[1] * quater[1] - 2 * quater[3] * quater[3];
		RM[2][1] = 2 * quater[2] * quater[3] + 2 * quater[1] * quater[0];

		RM[0][2] = 2 * quater[1] * quater[3] + 2 * quater[2] * quater[0];
		RM[1][2] = 2 * quater[2] * quater[3] - 2 * quater[1] * quater[0];
		RM[2][2] = 1 - 2 * quater[1] * quater[1] - 2 * quater[2] * quater[2];

		return true;
	}
}
coord rotation(coord P, array<array<double, 3>, 3>& RM, bool rotDir)
{
	coord R;
	if (!rotDir)
	{
		R[0] = P[0] * RM[0][0] + P[1] * RM[1][0] + P[2] * RM[2][0];
		R[1] = P[0] * RM[0][1] + P[1] * RM[1][1] + P[2] * RM[2][1];
		R[2] = P[0] * RM[0][2] + P[1] * RM[1][2] + P[2] * RM[2][2];
	}
	else
	{
	R[0] = P[0] * RM[0][0] + P[1] * RM[0][1] + P[2] * RM[0][2];
	R[1] = P[0] * RM[1][0] + P[1] * RM[1][1] + P[2] * RM[1][2];
	R[2] = P[0] * RM[2][0] + P[1] * RM[2][1] + P[2] * RM[2][2];
	}
	return R;
}
void intersection(coord V, coord R, array<double, 4>planeEquation, int& position, coord& interPoint)
{
	//rovnobezna, stejna
	double t;
	coord u;
	double par;

	u = { V[0] - R[0],V[1] - R[1],V[2] - R[2] };

	//primka je rovnobezna s rovinou, nebo v ni lezi?
	par = { u[0] * planeEquation[0] + u[1] * planeEquation[1] + u[2] * planeEquation[2] };
	if (epsCompar(par, 0.0))
	{
		//lezi v rovine
		if (epsCompar(V[0] * planeEquation[0] + V[1] * planeEquation[1] + V[2] * planeEquation[2] + planeEquation[3], 0.0))
		{
			position = 0;
		}
		//je rovnobezna
		else
		{
			position = -1;
		}
	}
	//ruznobezna->hledame prusecik
	else
	{
		t = -((planeEquation[0] * R[0] + planeEquation[1] * R[1] + planeEquation[2] * R[2] + planeEquation[3]) / (planeEquation[0] * u[0] + planeEquation[1] * u[1] + planeEquation[2] * u[2]));
		interPoint = { R[0] + u[0] * t,R[1] + u[1] * t ,R[2] + u[2] * t };
		position = 1;
	}
}
void normalize(coord& u)
{
	double lenght = sqrt(u[0] * u[0] + u[1] * u[1] + u[2] * u[2]);
	u[0] = u[0] / lenght;
	u[1] = u[1] / lenght;
	u[2] = u[2] / lenght;
}
double distancePP(coord A, coord B, coord& u)
{
	u = { A[0] - B[0],A[1] - B[1],A[2] - B[2] };
	double lenght = sqrt(u[0] * u[0] + u[1] * u[1] + u[2] * u[2]);
	u = { u[0] / lenght, u[1] / lenght ,u[2] / lenght };
	return lenght;
}
double distanceLP(coord L1, coord L2, coord S)
{
	coord u = { L1[0] - L2[0],L1[1] - L2[1],L1[2] - L2[2] };
	double t = (u[0] * (S[0] - L1[0]) + u[1] * (S[1] - L1[1]) + u[2] * (S[2] - L1[2])) / (u[0] * u[0] + u[1] * u[1] + u[2] * u[2]); //=0
	coord X = { L1[0] + t*u[0],L1[1] + t*u[1] ,L1[2] + t*u[2] };
	return distancePP(X, S, u);
}
void interLR(coord& interPoint1, coord& interPoint2, coord S, double r)
{
	coord u;
	//prusecik primky a kruznice
	if (epsCompar(interPoint1[0], interPoint2[0]) && epsCompar(interPoint1[1], interPoint2[1]) && epsCompar(interPoint1[2], interPoint2[2]))
	{
		//body se rovnaji, posuneme, aby vznikl vektor
		u = { 0.1,0.1,0.1 };

	}
	//prusecik primky a kruznice
	else
	{
		u = { interPoint1[0] - interPoint2[0],interPoint1[1] - interPoint2[1],interPoint1[2] - interPoint2[2] };
	}
	coord coef = { u[0] * u[0] + u[1] * u[1],(interPoint1[0] - S[0])* u[0] * 2 + (interPoint1[1] - S[1])* u[1] * 2,(interPoint1[0] - S[0])*(interPoint1[0] - S[0]) + (interPoint1[1] - S[1])*(interPoint1[1] - S[1]) - r*r };
	double dis = coef[1] * coef[1] - 4 * coef[0] * coef[2];
	double t1 = (-coef[1] - sqrt(dis)) / (2 * coef[0]);
	double t2 = (-coef[1] + sqrt(dis)) / (2 * coef[0]);

	
	interPoint2 = { interPoint1[0] + t2*u[0],interPoint1[1] + t2*u[1],interPoint1[2] };
	interPoint1 = { interPoint1[0] + t1*u[0],interPoint1[1] + t1*u[1],interPoint1[2] };
}
bool comparison(coord interPoint1, coord interPoint2, coord A, coord B)
{
	//lezi zadane dva body A,B mezi pruseciky
	for (int i = 0; i < 3; i++)
	{
		if ((interPoint1[i] < interPoint2[i]))
		{
			if (((A[i] > interPoint1[i]) || epsCompar(A[i], interPoint1[i])) && ((A[i]< interPoint2[i]) || epsCompar(A[i], interPoint2[i])) && ((B[i]>interPoint1[i]) || epsCompar(B[i], interPoint1[i])) && (B[i] < interPoint2[i]) || epsCompar(B[i], interPoint2[i]))
			{
			}
			else
			{
				return false;
			}
		}
		else if (interPoint1[i] > interPoint2[i])
		{
			if (((A[i] < interPoint1[i]) || epsCompar(A[i], interPoint1[i])) && ((A[i]> interPoint2[i]) || epsCompar(A[i], interPoint2[i])) && ((B[i]<interPoint1[i]) || epsCompar(B[i], interPoint1[i])) && (B[i] > interPoint2[i]) || epsCompar(B[i], interPoint2[i]))
			{
			}
			else
			{
				return false;
			}
		}
	}
	return true;
}
coord interLL(coord A, coord u, coord B, coord v, bool& e)
{
	e = false;
	double d = (v[1] - (v[0] * u[1] / u[0]));
	double s;
	coord interPoint;
	if (epsCompar(d, 0.0))
	{
		d = (v[2] - (v[0] * u[2] / u[0]));
		if (epsCompar(d, 0.0))
		{
			s = (A[2] - B[2] + ((u[2] / u[1])*(B[1] - A[1]))) / (v[2] - (v[1] * u[2] / u[1]));
			interPoint = { B[0] + s*v[0],B[1] + s*v[1] ,B[2] + s*v[2] };
		}
		else
		{
			s = (A[2] - B[2] + ((u[2] / u[0])*(B[0] - A[0]))) / (v[2] - (v[0] * u[2] / u[0]));
			interPoint = { B[0] + s*v[0],B[1] + s*v[1] ,B[2] + s*v[2] };
		}
	}
	else
	{
		s = (A[1] - B[1] + ((u[1] / u[0])*(B[0] - A[0]))) / (v[1] - (v[0] * u[1] / u[0]));
		interPoint = { B[0] + s*v[0],B[1] + s*v[1] ,B[2] + s*v[2] };
	}

	if (isnan(s) || isinf(s))
	{
		e = true;
	}
	return interPoint;
}
bool inputCheck(coord E, coord F, coord G, coord V, coord S, coord X, coord Y, array<double, 4>& SXY, array<double, 4>& EFG)
{
	bool err = false;
	coord u;

	// a) EFG tvori rovinu (jsou ruzne)
	EFG = planeEqv(E, F, G, err);
	if (err)
	{
		//cout << "CHYBA: Body E, F a G nelezi v rovine" << endl;
	}

	// b) SXY tvori rovinu (jsou ruzne)
	SXY = planeEqv(S, X, Y, err);
	if (err)
	{
		//cout << "CHYBA: Body X, Y a S nelezi v rovine" << endl;
	}

	//poloroviny
	double konstV = V[0] * SXY[0] + V[1] * SXY[1] + V[2] * SXY[2] + SXY[3];
	double konstE= E[0] * SXY[0] + E[1] * SXY[1] + E[2] * SXY[2] + SXY[3];
	//bool halfSpace = signbit(konstV); //true pokud zporn

	//porovnn norml rovin-> jestli jsou rovnobezne
	if (epsCompar(EFG[0], SXY[0]) && epsCompar(EFG[1], SXY[1]) && epsCompar(EFG[2], SXY[2]))
	{
		//kontrola jestli jsou ruzne
		if (epsCompar(EFG[3], SXY[3]))
		{
			//cout << "CHYBA: rovina SXY je shodna s rovinou EFG" << endl;
			err = true;
		}
	}
	else if (epsCompar(-EFG[0], SXY[0]) && epsCompar(-EFG[1], SXY[1]) && epsCompar(-EFG[2], SXY[2]))
	{
		if (epsCompar(-EFG[3], SXY[3]))
		{
			//cout << "CHYBA: rovina SXY je shodna s rovinou EFG" << endl;
			err = true;
		}
	}
	// c) X, Y lezi na kruznici:
	/*if (!epsCompar(distancePP(X, S, u), distancePP(Y, S, u)))
	{
		//cout << "CHYBA: Body X a Y nemaji stejnou vzdalenost od S, tedy netvori kruhovou podstavu" << endl;
		err = true;
	}
	*/
	// d) vrchol V nelezi v rovine podstavy
	if (epsCompar(konstV, 0.0))
	{
		//cout << "CHYBA: vrchol V lezi v rovine podstavy" << endl;
		err = true;
	}
	return err;
}
void rescale(coord V, coord S, coord& Sn, coord& Vn, double& r, double ra, coord& Xn, coord& Yn)
{
	coord u;
	coord v;
	coord w;


	coord X = { r,0.0,0.0 };
	coord Y = { -r,0.0,0.0 };
	distancePP(X, Y, u);
	distancePP(V, Y, v);
	distancePP(X, V, w);

	// n -kolmice na strany trojuhelniku -PPR -> smer dovnitr
	coord n1 = crossProductVect(u, v);
	n1 = crossProductVect(v, n1);

	v = { -v[0],-v[1],-v[2] };
	coord n2 = crossProductVect(v, w);
	n2 = crossProductVect(w, n2);

	u = { -u[0],-u[1],-u[2] };
	v = { -v[0],-v[1],-v[2] };

	coord n3 = crossProductVect(w, u);
	n3 = crossProductVect(u, n3);

	normalize(n1);
	normalize(n2);
	normalize(n3);

	//ve smeru kolmic posouvame bod na primce o vydalenost r
	X = { X[0] + ra*n2[0],X[1] + ra*n2[1],X[2] + ra*n2[2] };
	Y = { Y[0] + ra*n3[0],Y[1] + ra*n3[1],Y[2] + ra*n3[2] };
	V = { V[0] + ra*n1[0],V[1] + ra*n1[1],V[2] + ra*n1[2] };

	//pruseciky novych primek = nove body zmenseneho kuzelu
	bool e;
	Xn = interLL(X, w, Y, u,e);
	Yn = interLL(V, v, Y, u,e);
	Vn = interLL(V, v, X, w,e);
	Sn = { Yn[0] + ((Xn[0] - Yn[0]) / 2), Yn[1] + ((Xn[1] - Yn[1]) / 2), Xn[2] };
	r = distancePP(Sn, Xn, u);
}
int main()
{
	// 1) ZADANI VSTUPNICH DAT:  vrchol+ stred podstavy + 2 ruzne body na kruznici
	time_point<steady_clock> start, finish;
	bool result = true;
	//vrchol
	coord V = { 1.0,1.0,4.0 };

	//stred kruznice
	coord So = { 0.0,0.0,0.0 };

	//body na kruznici
	coord X = { -0.66,-2.8,0.86};
	coord Y = { -2.0,-2.0,1.0 };

	////cout << "puvodni souradnice kuzelu:" << endl;
	coord E = {4.0,4.0,1.0};
	coord F = { 4.0,-4.0,1.0 };
	coord G = {-3.0,0.0,5.0};
 
	// 2) KONTROLA VSTUPNICH DAT

	array<double, 4> SXY,EFG;
	//bool halfSpace;

	if (inputCheck(E, F, G,V, So, X, Y, SXY,EFG))
	{
		//cout << "Chyba ve vstupnich datech." << endl;
	}
	else
	{
		// 3) TRANSORMACE => S=[0,0,0], kruznice v rovine XY
		// a) posunuti => vsechny body -S
		for (int i = 0; i < 3; i++)
		{
			V[i] = V[i] - So[i];
			X[i] = X[i] - So[i];
			Y[i] = Y[i] - So[i];
			E[i] = E[i] - So[i];
			F[i] = F[i] - So[i];
			G[i] = G[i] - So[i];
		}
		coord S = { 0.0,0.0,0.0 };

		// b) rotace 
		bool err;
		SXY = planeEqv(S, X, Y, err);
		array<array<double, 3>, 3> RM;

		coord z0 = { 0.0,0.0,1.0};

		bool rotate = getRotMatrix(z0, {SXY[0], SXY[1], SXY[2]}, RM);
		bool rotDir = signbit(SXY[2]);
		if (rotate)
		{
			V = rotation(V, RM, rotDir);
			X = rotation(X, RM, rotDir);
			Y = rotation(Y, RM, rotDir);
			E = rotation(E, RM, rotDir);
			F = rotation(F, RM, rotDir);
			G = rotation(G, RM, rotDir);
		}

		start = high_resolution_clock::now();
		// 4) ZADAVANI USECEK
		coord A = {0.19,0.19,0.76};
		coord B = { 0.38,0.38,1.51 };
		double ra = 0.2;

		//4.1) kontrola
		if (epsCompar(A[0], B[0]) && epsCompar(A[1], B[1]) && epsCompar(A[2], B[2]))
		{
			//cout << "CHYBA: zadane body netvori usecku" << endl;
		}
		else
		{
			// 4.2) transformace usecky
			for (int i = 0; i < 3; i++)
			{
				A[i] = A[i] - So[i];
				B[i] = B[i] - So[i];
			}
			if (rotate)
			{
				A = rotation(A, RM, rotDir);
				B = rotation(B, RM, rotDir);
			}
			//cout << "rotovane  souradnice:" << endl;
			//cout << "X: " << X[0] << " " << X[1] << " " << X[2] << endl;
			//cout << "Y: " << Y[0] << " " << Y[1] << " " << Y[2] << endl;
			//cout << "V: " << V[0] << " " << V[1] << " " << V[2] << endl;
			//cout << "S: " << S[0] << " " << S[1] << " " << S[2] << endl;
			//cout << "A: " << A[0] << " " << A[1] << " " << A[2] << endl;
			//cout << "B: " << B[0] << " " << B[1] << " " << B[2] << endl;
			//cout << "E: " << E[0] << " " << E[1] << " " << E[2] << endl;
			//cout << "F: " << F[0] << " " << F[1] << " " << F[2] << endl;
			//cout << "G: " << G[0] << " " << G[1] << " " << G[2] << endl;
			//cout << endl;

			// 5) ZMENSENI KUZELU O POLOMER
			coord u, Sn, Vn, Xn, Yn;
			double r = distancePP(S, X, u);
			if (ra < min({ distancePP(V, { r,0.0,0.0 }, u), distancePP(V, { -r,0.0,0.0 }, u), 2 * r }))
			{
				EFG = planeEqv(E, F, G, err);
				//posunut ezn roviny o polomr
				coord Econ = { E[0] + ra*EFG[0],E[1] + ra*EFG[1],E[2] + ra*EFG[2] };
				bool sigECon = signbit(Econ[0] * EFG[0] + Econ[1] * EFG[1] + Econ[2] * EFG[2] + EFG[3]);
				bool sigSCon = signbit(S[0] * EFG[0] + S[1] * EFG[1] + S[2] * EFG[2] + EFG[3]);

				if (sigECon == sigSCon)
				{
					E = Econ;
					F = { F[0] + ra*EFG[0],F[1] + ra*EFG[1],F[2] + ra*EFG[2] };
					G = { G[0] + ra*EFG[0],G[1] + ra*EFG[1],G[2] + ra*EFG[2] };
				}
				else
				{
					E = { E[0] - ra*EFG[0],E[1] - ra*EFG[1],E[2] - ra*EFG[2] };
					F = { F[0] - ra*EFG[0],F[1] - ra*EFG[1],F[2] - ra*EFG[2] };
					G = { G[0] - ra*EFG[0],G[1] - ra*EFG[1],G[2] - ra*EFG[2] };
				}
				//cout << "posunuta rovina:" << endl;
				//cout << "E: " << E[0] << " " << E[1] << " " << E[2] << endl;
				//cout << "F: " << F[0] << " " << F[1] << " " << F[2] << endl;
				//cout << "G: " << G[0] << " " << G[1] << " " << G[2] << endl;
				//A a B ve spravne polorovine vuci posunute rovine
				bool error;
				EFG = planeEqv(E, F, G, error);
				if (!(signbit(A[0] * EFG[0] + A[1] * EFG[1] + A[2] * EFG[2] + EFG[3]) == sigSCon&& signbit(B[0] * EFG[0] + B[1] * EFG[1] + B[2] * EFG[2] + EFG[3]) == sigSCon))
				{
					result = false;
				}

				if (result)
				{
					rescale(V, S, Sn, Vn, r, ra, Xn, Yn);
				}
				//cout << "zmenseny kuzel:" << endl;
				//cout << "X: " << Xn[0] << " " << Xn[1] << " " << Xn[2] << endl;
				//cout << "Y: " << Yn[0] << " " << Yn[1] << " " << Yn[2] << endl;
				//cout << "V: " << Vn[0] << " " << Vn[1] << " " << Vn[2] << endl;
				//cout << "S: " << Sn[0] << " " << Sn[1] << " " << Sn[2] << endl;
				if (signbit(V[2]))
				{
					if (!(Vn[2] < Sn[2]))
					{
						result = false;
					}
					else if (Xn[0] < Yn[0])
					{
						result = false;
					}
				}
				else
				{
					if (Vn[2] < Sn[2])
					{
						result = false;
					}
					else if (Xn[0] < Yn[0])
					{
						result = false;
					}
				}
			}
			else
			{
				result = false;
			}

				// posunuti zpet do poc. souradnic
				V = { Vn[0] - Sn[0],Vn[1] - Sn[1] ,Vn[2] - Sn[2] };
				A = { A[0] - Sn[0],A[1] - Sn[1],A[2] - Sn[2] };
				B = { B[0] - Sn[0],B[1] - Sn[1],B[2] - Sn[2] };
				X = { X[0] - Sn[0],X[1] - Sn[1],X[2] - Sn[2] };
				Y = { Y[0] - Sn[0],Y[1] - Sn[1],Y[2] - Sn[2] };
				E = { E[0] - Sn[0],E[1] - Sn[1],E[2] - Sn[2] };
				F = { F[0] - Sn[0],F[1] - Sn[1],F[2] - Sn[2] };
				G = { G[0] - Sn[0],G[1] - Sn[1],G[2] - Sn[2] };

				//cout << "X: " << X[0] << " " << X[1] << " " << X[2] << endl;
				//cout << "Y: " << Y[0] << " " << Y[1] << " " << Y[2] << endl;
				//cout << "V: " << V[0] << " " << V[1] << " " << V[2] << endl;
				//cout << "A: " << A[0] << " " << A[1] << " " << A[2] << endl;
				//cout << "B: " << B[0] << " " << B[1] << " " << B[2] << endl;
				//cout << "S: " << S[0] << " " << S[1] << " " << S[2] << endl;
			}
			// 4) VYPOCET
			array<double, 4> xyPlane = { 0.0,0.0,1.0,0.0 };
			
			if (signbit(A[2]) != signbit(V[2]))
			{
				result = false;
			}
			if (signbit(B[2]) != signbit(V[2]))
			{
				result = false;
			}

			//parametry pruseciku
			int pos; coord interPoint1, interPoint2;

			//vrcholova rovina - pokud prochazi vrcholem, tak prunik primky s podstavou !!!!!
			bool error = false;
			coord u;
			array<double, 4> vertexPlane = planeEqv(A, B, V, error);
			double r = distancePP(S, X, u);


			if (error == true && result)
			{
				intersection(A, B, xyPlane, pos, interPoint1);

				if (pos == -1)
				{
					//rovnobezna s podstavou+ prochazi vrcholem -> nema prusecik
					result = false;
				}
				else if (pos == 1)
				{
					//1 prusecik je vrchol, 2. je prusecik interPoint => vypocitavame, jestli lezi uvnitr kruznice (vzdalenost 2 bodu)
					coord v;
					if ((distancePP(interPoint1, S, v) < r) || epsCompar(distancePP(interPoint1, S, v), r))
					{
						//OK
						//porovnani, jestli lezi uvnitr pruseciku
						result = comparison(interPoint1, V, A, B);
					}
					else
					{
						result = false;
					}
				}
			}

			//pudorysny stopnik
			else if (result)
			{
				distancePP(A, B, u);
				coord R = { A[0] + u[0],A[1] + u[1],A[2] + u[2] };
				if (epsCompar(R[0], B[0]) && epsCompar(R[1], B[1]) && epsCompar(R[2], B[2]))
				{
					R = { R[0] + u[0],R[1] + u[1],R[2] + u[2] };
				}
				intersection(B, A, xyPlane, pos, interPoint1);

				if (pos == 0)
				{
					if (distanceLP(A, B, S) < r || epsCompar(distanceLP(A, B, S), r))
					{
						interPoint1 = A;
						interPoint2 = B;
						interLR(interPoint1, interPoint2, S, r);
						//POROVNN
						result = comparison(interPoint1, interPoint2, A, B);
					}
					else
					{
						result = false;
					}

				}
				else if (pos == -1)
				{
					//rovnobezna s podstavou !!!! 
					//prepocitame polomer a stred noveho (posunuteho kruhu)
					double dist = distancePP(S, V, u);
					double k = A[2] / u[2];
					coord Sn = { S[0] + u[0] * k, S[1] + u[1] * k, S[2] + u[2] * k };
					double rn = (r*distancePP(Sn, V, u)) / dist;
					// pruseciky nove kruznice s primkou
					if (distanceLP(A, B, Sn) < rn || epsCompar(distanceLP(A, B, Sn), rn))
					{
						interLR(interPoint1, interPoint2, Sn, rn);
						//POROVNANI
						result = comparison(interPoint1, interPoint2, A, B);
					}
					else
					{
						result = false;
					}

				}
				else if (pos == 1)
				{
					//pudorysny stopnik AB - interPoint -V ==vrcholova rovina
					intersection(V, R, xyPlane, pos, interPoint2);
					//podle vydalenosti primky od bodu 
					double dist = distanceLP(interPoint1, interPoint2, S);
					if (dist < r)
					{
						//prusecik kruznice a primky
						interLR(interPoint1, interPoint2, S, r);
						//prusecik 2 primek
						coord v;
						distancePP(interPoint1, V, v);
						interPoint1 = interLL(V, v, A, u,err);
						if (err)
						{
							intersection(V, A, xyPlane, pos, interPoint1);
							if (!distancePP(interPoint1, S, v) < r)
							{
								result = false;
							}
						}

						distancePP(interPoint2, V, v);
						interPoint2 = interLL(V, v, A, u,err);
						if (err)
						{
							intersection(V, A, xyPlane, pos, interPoint2);
							if (!distancePP(interPoint2, S, v) < r)
							{
								result = false;
							}
						}


						//POROVNANI
						if (signbit(V[2]) && ((interPoint1[2] < V[2]) || (interPoint2[2] < V[2]) || (interPoint2[2] > S[2]) || (interPoint1[2] > S[2])))
						{
							if ((interPoint1[2] < V[2]) || (interPoint1[2] > S[2]))
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint1);
							}
							else
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint2);
							}							
						}
						else if ((!signbit(V[2])) && ((interPoint1[2] > V[2]) || (interPoint2[2] > V[2]) || (interPoint2[2] < S[2]) || (interPoint1[2] < S[2])))
						{
							if ((interPoint1[2] > V[2]) || (interPoint1[2] < S[2]))
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint1);
							}
							else
							{
								intersection(interPoint1, interPoint2, xyPlane, pos, interPoint2);
							}
						}
						else
						{
						result = comparison(interPoint1, interPoint2, A, B);
						}
					}
					else
					{
						result = false;
					}

				}
			}
		
		if (result)
		{
			cout << "usecka lezi uvnitr zadaneho kuzelu" << endl;
		}
		else
		{
			cout << "CHYBA: cast, nebo cela zadana usecka je mimo kuzel" << endl;
		}
		finish = high_resolution_clock::now();

		auto sec = duration_cast<microseconds>(finish - start);
		cout << "time: " << sec.count() << " us" << endl;
		cin.get();
	}
	return 0;
}

